@@ -7,7 +7,7 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-__version__ = '2.0.0'
+__version__ = '2.0.1'
 __author__ = "Davide Brunato"
 __contact__ = "brunato@sissa.it"
 __copyright__ = "Copyright 2018-2020, SISSA"
@@ -24,7 +24,7 @@ from .xpath_context import XPathContext, XPathSchemaContext
 from .xpath_nodes import AttributeNode, TextNode, TypedAttribute, TypedElement, NamespaceNode
 from .xpath_token import XPathToken
 from .xpath1_parser import XPath1Parser
-from .xpath2_constructors import XPath2Parser
+from .xpath2_parser import XPath2Parser
 from .xpath_selectors import select, iter_select, Selector
 from .schema_proxy import AbstractSchemaProxy
 
