@@ -12,7 +12,7 @@ import unittest
 import re
 from collections import namedtuple
 
-from elementpath.tdop_parser import symbol_to_identifier, Parser, MultiLabel
+from elementpath.tdop import symbol_to_identifier, Parser, MultiLabel
 
 
 FakeToken = namedtuple('Token', 'symbol pattern')
@@ -78,12 +78,11 @@ class TdopParserTest(unittest.TestCase):
     def test_create_tokenizer_method(self):
         pattern = Parser.create_tokenizer(create_fake_tokens(['(name)', 'call', '+']))
         self.assertEqual(pattern.pattern, r"""
-            ('[^']*' | "[^"]*" | (?:\d+|\.\d+)(?:\.\d*)  # Literals (strings and numbers)
-            ?(?:[Ee][+-]?\d+)?) |
-            (call|[+]) |                                  # Symbol's patterns
-            ([A-Za-z0-9_]+) |                                       # Names
-            (\S) |                                       # Unexpected characters
-            \s+                                          # Skip extra spaces
+            ('[^']*'|"[^"]*"|(?:\d+|\.\d+)(?:\.\d*)?(?:[Ee][+-]?\d+)?) |       # Literals
+            (call|[+]) |  # Symbols
+            ([A-Za-z0-9_]+) |       # Names
+            (\S) |       # Unknown symbols
+            \s+          # Skip extra spaces
         """)
 
         with self.assertRaises(ValueError):
@@ -94,7 +93,7 @@ class TdopParserTest(unittest.TestCase):
             ['(name)', 'call', '+', '{http://www.w3.org/2000/09/xmldsig#}CryptoBinary']
         ))
         self.assertTrue(
-            pattern.pattern.split('\n')[3].strip().startswith(
+            pattern.pattern.split('\n')[2].strip().startswith(
                 r"({http://www.w3.org/2000/09/xmldsig\#}"
             )
         )
