@@ -64,8 +64,7 @@ XML_DATA_TEST = """
 class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
     def setUp(self):
-        self.parser = XPath1Parser(self.namespaces, self.variables, strict=True)
-        # self.token = XPath1Parser.symbol_table['(name)'](self.parser, 'test')
+        self.parser = XPath1Parser(self.namespaces, strict=True)
 
     #
     # Test methods
@@ -103,6 +102,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             "employee[@secretary and @assistant]",
             ['employee', '[', '@', 'secretary', '', 'and', '', '@', 'assistant', ']']
         )
+        self.check_tokenizer('/root/a/true()', ['/', 'root', '/', 'a', '/', 'true', '(', ')'])
 
         # additional tests from Python XML etree test cases
         self.check_tokenizer("{http://spam}egg", ['{', 'http', ':', '//', 'spam', '}', 'egg'])
@@ -203,30 +203,29 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
     def test_parser_position(self):
         self.assertEqual(self.parser.position, (1, 1))
 
-        try:
-            self.assertIsNone(self.parser.parse('false(x)'))
-        except SyntaxError as err:
-            self.assertIn('line 1, column 7', str(err))
+        with self.assertRaises(SyntaxError) as ctx:
+            self.parser.parse('child::node())')
+        self.assertIn('line 1, column 14', str(ctx.exception))
 
-        try:
-            self.assertIsNone(self.parser.parse(' false(x)'))
-        except SyntaxError as err:
-            self.assertIn('line 1, column 8', str(err))
+        with self.assertRaises(SyntaxError) as ctx:
+            self.parser.parse(' child::node())')
+        self.assertIn('line 1, column 15', str(ctx.exception))
 
-        try:
-            self.assertIsNone(self.parser.parse(' false( x)'))
-        except SyntaxError as err:
-            self.assertIn('line 1, column 9', str(err))
+        with self.assertRaises(SyntaxError) as ctx:
+            self.parser.parse(' child::node( ))')
+        self.assertIn('line 1, column 16', str(ctx.exception))
 
-        try:
-            self.assertIsNone(self.parser.parse('^'))
-        except SyntaxError as err:
-            self.assertIn('line 1, column 1', str(err))
+        with self.assertRaises(SyntaxError) as ctx:
+            self.parser.parse(' child::node() )')
+        self.assertIn('line 1, column 16', str(ctx.exception))
 
-        try:
-            self.assertIsNone(self.parser.parse(' ^'))
-        except SyntaxError as err:
-            self.assertIn('line 1, column 2', str(err))
+        with self.assertRaises(SyntaxError) as ctx:
+            self.parser.parse('^)')
+        self.assertIn('line 1, column 1', str(ctx.exception))
+
+        with self.assertRaises(SyntaxError) as ctx:
+            self.parser.parse(' ^)')
+        self.assertIn('line 1, column 2', str(ctx.exception))
 
     def test_wrong_syntax(self):
         self.wrong_syntax('')
@@ -246,9 +245,9 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
     # XPath expression tests
     def test_node_selection(self):
-        self.check_value("mars", [])
-
         root = self.etree.XML('<A><B1/><B2/><B3/><B2/></A>')
+        self.check_value("mars", MissingContextError)
+        self.check_value("mars", [], context=XPathContext(root))
         self.check_value("B1", [root[0]], context=XPathContext(root))
         self.check_value("B2", [root[1], root[3]], context=XPathContext(root))
         self.check_value("B4", [], context=XPathContext(root))
@@ -269,15 +268,16 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
         # Test evaluate method
         self.check_value("fn:true()", True)
-        self.check_value("fx:true()", KeyError)
+        self.check_value("fx:true()", NameError)
         self.check_value("tst:B1", [root[0]], context=XPathContext(root))
         self.check_value("tst:B2", [root[1], root[3]], context=XPathContext(root))
+        self.check_value("tst:B1:B2", NameError)
 
         self.check_selector("./tst:B1", root, [root[0]], namespaces=namespaces)
         self.check_selector("./tst:*", root, root[:], namespaces=namespaces)
         self.wrong_syntax("./tst:1")
-        self.wrong_syntax("./fn:A")
-        self.wrong_syntax("./xs:true()")
+        self.check_value("./fn:A", MissingContextError)
+        self.wrong_type("./xs:true()")
 
         # Namespace wildcard works only for XPath > 1.0
         if self.parser.version == '1.0':
@@ -301,9 +301,11 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
                         "(string ({ ('http://www.w3.org/2001/XMLSchema') (unknown)))")
         self.wrong_syntax("{%s" % XSD_NAMESPACE)
         self.wrong_syntax("{%s}1" % XSD_NAMESPACE)
-        self.wrong_syntax("{%s}alpha" % XPATH_FUNCTIONS_NAMESPACE)
-
         self.check_value("{%s}true()" % XPATH_FUNCTIONS_NAMESPACE, True)
+
+        name = '{%s}alpha' % XPATH_FUNCTIONS_NAMESPACE
+        self.check_value(name, name)  # it's not an error to use 'fn' namespace for a name
+
         self.parser.strict = True
         self.wrong_syntax('{%s}string' % XSD_NAMESPACE)
 
@@ -341,30 +343,29 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_selector("node()", self.etree.XML('<author>Dickens</author>'), ['Dickens'])
         self.check_selector("text()", self.etree.XML('<author>Dickens</author>'), ['Dickens'])
 
-        root = self.etree.XML('<author>Dickens</author>')
+        document = self.etree.parse(io.StringIO('<author>Dickens</author>'))
+        root = document.getroot()
         if self.etree is not lxml_etree:
+            # self.check_value("//self::node()", [document, root, 'Dickens'], context=context)
             # Skip lxml test because lxml's XPath doesn't include document root
-            self.check_selector("//self::node()", root, [root, root, 'Dickens'])
+            self.check_selector("//self::node()", document, [document, root, 'Dickens'])
         self.check_selector("//self::text()", root, ['Dickens'])
 
     def test_node_set_id_function(self):
         # XPath 1.0 id() function: https://www.w3.org/TR/1999/REC-xpath-19991116/#function-id
         root = self.etree.XML('<A><B1 xml:id="foo"/><B2/><B3 xml:id="bar"/><B4 xml:id="baz"/></A>')
-        if self.parser.version == '1.0':
-            self.check_selector('id("foo")', root, [root[0]])
-        else:
-            self.check_selector('id("foo")', root, ValueError)
+        self.check_selector('id("foo")', root, [root[0]])
 
     def test_node_set_functions(self):
         root = self.etree.XML('<A><B1><C1/><C2/></B1><B2/><B3><C3/><C4/><C5/></B3></A>')
         context = XPathContext(root, item=root[1], size=3, position=3)
-        self.check_value("position()", 0)
+        self.check_value("position()", MissingContextError)
         self.check_value("position()", 3, context=context)
-        self.check_value("position()<=2", True)
+        self.check_value("position()<=2", MissingContextError)
         self.check_value("position()<=2", False, context=context)
         self.check_value("position()=3", True, context=context)
         self.check_value("position()=2", False, context=context)
-        self.check_value("last()", 0)
+        self.check_value("last()", MissingContextError)
         self.check_value("last()", 3, context=context)
         self.check_value("last()-1", 2, context=context)
 
@@ -388,7 +389,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
                             namespaces={'tst': "http://xpath.test/ns", '': ''})
 
     def test_string_function(self):
-        self.check_value("string(10.0)", '10.0')
+        self.check_value("string(10.0)", '10')
         if self.parser.version == '1.0':
             self.wrong_syntax("string(())")
         else:
@@ -469,10 +470,15 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         variables = {'ups1': root[0], 'ups2': root[1], 'ups3': root[2]}
         self.check_selector('string($ups1/power)', root, '40kW', variables=variables)
 
-        self.check_value('$word', 'alpha')
-        self.wrong_syntax('$eg:word', 'variable reference requires a simple reference name')
-        self.wrong_syntax('${http://xpath.test/ns}word',
-                          "unexpected symbol '{' after $ variable reference")
+        context = XPathContext(root, variables=self.variables)
+        self.check_value('$word', 'alpha', context)
+        self.wrong_syntax('${http://xpath.test/ns}word', 'XPST0003')
+
+        if self.parser.version == '1.0':
+            self.wrong_syntax('$eg:word', 'variable reference requires a simple reference name')
+        else:
+            context = XPathContext(root, variables={'eg:color': 'purple'})
+            self.check_value('$eg:color', 'purple', context)
 
     def test_substring_function(self):
         root = self.etree.XML(XML_GENERIC_TEST)
@@ -507,6 +513,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_value('fn:substring("12345", 0 div 0E0, 3)', '')
             self.check_value('fn:substring("12345", 1, 0 div 0E0)', '')
             self.check_value('fn:substring((), 1, 3)', '')
+
+            self.check_value('fn:substring("12345", -42, 1 div 0)', ZeroDivisionError)
             self.check_value('fn:substring("12345", -42, 1 div 0E0)', '12345')
             self.check_value('fn:substring("12345", -1 div 0E0, 1 div 0E0)', '')
 
@@ -515,8 +523,11 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_value('fn:substring(("alpha"), 1, (3))', 'alp')
             self.wrong_type('fn:substring(("alpha"), (1, 2), 3)')
             self.wrong_type('fn:substring(("alpha", "beta"), 1, 3)')
+
             self.parser.compatibility_mode = True
             self.check_value('fn:substring(("alpha", "beta"), 1, 3)', 'alp')
+            self.check_value('fn:substring("12345", -42, 1 div 0E0)', '12345')
+            self.check_value('fn:substring("12345", -1 div 0E0, 1 div 0E0)', '')
             self.parser.compatibility_mode = False
 
     def test_starts_with_function(self):
@@ -570,9 +581,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_selector("//b[concat(., '_foo') = 'some content_foo']", root, [root[0][0]])
         self.check_selector("//b[concat(., '_fo') = 'some content_foo']", root, [])
         self.check_selector("//none[concat(., '_fo') = 'some content_foo']", root, [])
-        self.wrong_syntax("concat()")
+        self.wrong_type("concat()", 'XPST0017')
 
-        self.wrong_syntax("concat()")
         if self.parser.version == '1.0':
             self.wrong_syntax("concat((), (), ())")
         else:
@@ -725,7 +735,9 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_value("1 and 1", True)
         self.check_value("1 and 'jupiter'", True)
         self.check_value("0 and 'mars'", False)
-        self.check_value("1 and mars", False)
+        self.check_value("1 and mars", MissingContextError)
+        context = XPathContext(self.etree.XML('<root/>'))
+        self.check_value("1 and mars", False, context)
 
     def test_logical_or_operator(self):
         self.check_value("false() or true()", True)
@@ -754,10 +766,15 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_value("5 >= 9", False)
         self.check_value("5 > 3", True)
         self.check_value("5 < 20.0", True)
-        self.check_value("false() = 1", False)
-        self.check_value("0 = false()", True)
         self.check_value("2 * 2 = 4", True)
 
+        if self.parser.version == '1.0':
+            self.check_value("false() = 1", False)
+            self.check_value("0 = false()", True)
+        else:
+            self.wrong_type("false() = 1")
+            self.wrong_type("0 = false()")
+
     def test_comparison_of_sequences(self):
         root = self.etree.XML('<table>'
                               '    <unit id="1"><cost>50</cost></unit>'
@@ -789,7 +806,9 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
     def test_div_operator(self):
         self.check_value("5 div 2", 2.5)
         self.check_value("0 div 2", 0.0)
-        if self.parser.version > '1.0':
+        if self.parser.version == '1.0':
+            self.check_value("10div 3", SyntaxError)  # TODO: accepted syntax in XPath 1.0
+        else:
             self.check_value("() div 2")
 
     def test_numerical_add_operator(self):
@@ -806,7 +825,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_value("3 + 'alpha'", float('nan'))
         else:
             self.check_selector("/values/a + 2", root, TypeError)
-            self.check_value("/values/b + 2", TypeError, context=XPathContext(root))
+            self.check_value("/values/b + 2", ValueError, context=XPathContext(root))
             self.wrong_type("+'alpha'")
             self.wrong_type("3 + 'alpha'")
 
@@ -826,7 +845,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_value("3 - 'alpha'", float('nan'))
         else:
             self.check_selector("/values/a - 2", root, TypeError)
-            self.check_value("/values/b - 2", TypeError, context=XPathContext(root))
+            self.check_value("/values/b - 2", ValueError, context=XPathContext(root))
             self.wrong_type("-'alpha'")
             self.wrong_type("3 - 'alpha'")
 
@@ -889,7 +908,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
     def test_sum_function(self):
         root = self.etree.XML(XML_DATA_TEST)
-        self.check_value("sum($values)", 35)
+        context = XPathContext(root, variables=self.variables)
+        self.check_value("sum($values)", 35, context)
         if self.parser.version == '1.0':
             self.wrong_syntax("sum(())")
         else:
@@ -940,11 +960,11 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
     def test_context_variables(self):
         root = self.etree.XML('<A><B1><C/></B1><B2/><B3><C1/><C2/></B3></A>')
         context = XPathContext(root, variables={'alpha': 10, 'id': '19273222'})
-        self.check_value("$alpha", None)  # Do not raise if the dynamic context is None
+        self.check_value("$alpha", MissingContextError)
         self.check_value("$alpha", 10, context=context)
         self.check_value("$beta", NameError, context=context)
         self.check_value("$id", '19273222', context=context)
-        self.wrong_syntax("$id()")
+        self.wrong_type("$id()")
 
     def test_path_step_operator(self):
         root = self.etree.XML('<A><B1><C1/></B1><B2/><B3><C1/><C2/></B3></A>')
@@ -969,8 +989,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_selector('/A/child::C1', root, [])
 
         if self.parser.version == '1.0':
-            self.wrong_syntax('/true()')
-            self.wrong_syntax('/A/true()')
+            self.wrong_type('/true()')
+            self.wrong_type('/A/true()')
             self.wrong_syntax('/|')
         else:
             self.check_value('/true()', [True], context=XPathContext(root))
@@ -1121,15 +1141,6 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_selector('/A/namespace::*', root, expected=set(namespaces),
                             namespaces=namespaces[-1:])
 
-        if self.etree is lxml_etree:
-            self.check_selector('/A/namespace::*', root, expected=set(namespaces))
-
-            root = self.etree.XML('<tst:A xmlns:tst="http://xpath.test/ns" '
-                                  'xmlns="http://xpath.test/ns"><B1/></tst:A>')
-            namespaces = namespaces + [(None, 'http://xpath.test/ns')]
-            self.check_selector('/tst:A/namespace::*', root, set(namespaces),
-                                namespaces=namespaces[-2:-1])
-
     def test_parent_shortcut_and_axis(self):
         root = self.etree.XML(
             '<A><B1><C1/></B1><B2/><B3><C1/><C2/></B3><B4><C3><D1/></C3></B4></A>')
@@ -1188,10 +1199,11 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.check_selector('/root/a/position()', root, [1, 2])
         else:
             # Functions are not allowed after path step in XPath 1.0
-            self.wrong_syntax('/root/a/true()')
+            self.wrong_type('/root/a/true()')
 
     def test_unknown_axis(self):
-        self.check_value('unknown::node()', NameError)
+        self.wrong_name('unknown::node()', 'XPST0010')
+        self.wrong_name('A/unknown::node()', 'XPST0010')
 
     def test_predicate(self):
         root = self.etree.XML('<A><B1><C1/><C2/><C3/></B1><B2><C1/><C2/><C3/><C4/></B2></A>')
@@ -1230,6 +1242,10 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
 
     def test_parenthesized_expression(self):
         self.check_value('(6 + 9)', 15)
+        if self.parser.version == '1.0':
+            self.check_value('()', ElementPathSyntaxError)
+        else:
+            self.check_value('()', [])
 
     def test_union(self):
         root = self.etree.XML(
@@ -1266,6 +1282,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             # XPath 1.0 ignores the default namespace declarations
             self.check_selector("name(B1)", root, '', namespaces={'': "http://xpath.test/ns"})
 
+    @unittest.skipIf(lxml_etree is None, 'lxml is not installed')
     def test_issue_25_with_count_function(self):
         root = lxml_etree.fromstring("""
             <line>
@@ -1334,6 +1351,21 @@ class LxmlXPath1ParserTest(XPath1ParserTest):
             else:
                 self.assertTrue(expected(results))
 
+    def test_namespace_axis(self):
+        root = self.etree.XML('<A xmlns:tst="http://xpath.test/ns"><tst:B1/></A>')
+        namespaces = list(self.parser.DEFAULT_NAMESPACES.items()) \
+            + [('tst', 'http://xpath.test/ns')]
+        self.check_selector('/A/namespace::*', root, expected=set(namespaces),
+                            namespaces=namespaces[-1:])
+
+        self.check_selector('/A/namespace::*', root, expected=set(namespaces))
+
+        root = self.etree.XML('<tst:A xmlns:tst="http://xpath.test/ns" '
+                              'xmlns="http://xpath.test/ns"><B1/></tst:A>')
+        namespaces.append((None, 'http://xpath.test/ns'))
+        self.check_selector('/tst:A/namespace::*', root, set(namespaces),
+                            namespaces=namespaces[-2:-1])
+
 
 if __name__ == '__main__':
     unittest.main()
