@@ -27,6 +27,7 @@ import math
 import os
 import platform
 import time
+from decimal import Decimal
 
 try:
     import lxml.etree as lxml_etree
@@ -43,7 +44,7 @@ else:
 from elementpath import *
 from elementpath.namespaces import XSI_NAMESPACE
 from elementpath.datatypes import DateTime, Date, Time, Timezone, \
-    DayTimeDuration, YearMonthDuration, UntypedAtomic
+    DayTimeDuration, YearMonthDuration, QName, UntypedAtomic
 
 try:
     from tests import test_xpath1_parser
@@ -68,7 +69,7 @@ except ImportError:
 class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
     def setUp(self):
-        self.parser = XPath2Parser(namespaces=self.namespaces, variables=self.variables)
+        self.parser = XPath2Parser(namespaces=self.namespaces)
 
         # Make sure the tests are repeatable.
         env_vars_to_tweak = 'LC_ALL', 'LANG'
@@ -92,7 +93,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         # Test cases taken from https://www.w3.org/TR/xquery-operators/#numeric-value-functions
         self.check_value("abs(10.5)", 10.5)
         self.check_value("abs(-10.5)", 10.5)
-        self.check_value("abs(())", [])
+        self.check_value("abs(())")
 
         root = self.etree.XML('<root>-10</root>')
         context = XPathContext(root, item=float('nan'))
@@ -102,7 +103,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value("abs(.)", 10, context=context)
         context = XPathContext(root=self.etree.XML('<root>foo</root>'))
 
-        self.wrong_type('abs("10")', 'FORG0006', 'Invalid argument type')
+        self.wrong_type('abs("10")', 'XPTY0004', 'invalid argument type')
 
         with self.assertRaises(ValueError) as err:
             self.check_value("abs(.)", 10, context=context)
@@ -110,7 +111,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.assertIn('Invalid string value', str(err.exception))
 
     def test_round_half_to_even_function(self):
-        self.check_value("round-half-to-even(())", [])
+        self.check_value("round-half-to-even(())")
         self.check_value("round-half-to-even(0.5)", 0)
         self.check_value("round-half-to-even(1.5)", 2)
         self.check_value("round-half-to-even(2.5)", 2)
@@ -122,7 +123,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         context = XPathContext(root, item=float('nan'))
         self.check_value("round-half-to-even(.)", float('nan'), context=context)
 
-        self.wrong_type('round-half-to-even("wrong")', 'FORG0006', 'Invalid argument type')
+        self.wrong_type('round-half-to-even("wrong")', 'XPTY0004', 'invalid argument type')
 
     def test_sum_function(self):
         self.check_value("sum((10, 15, 6, -2))", 29)
@@ -139,7 +140,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
                          context=context)
         root_token = self.parser.parse("fn:avg(($d1, $seq3))")
         self.assertRaises(TypeError, root_token.evaluate, context=context)
-        self.check_value("fn:avg(())", [])
+        self.check_value("fn:avg(())")
         self.wrong_type("fn:avg('10')", 'FORG0006')
         self.check_value("fn:avg($seq3)", 4.0, context=context)
 
@@ -165,6 +166,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
         root = self.etree.XML('<a><b>1</b><b>9</b></a>')
         self.check_selector('max(/a/b/number(text()))', root, 9)
+        self.check_selector('max(/a/b)', root, 9)
 
     def test_min_function(self):
         self.check_value("fn:min((3,4,5))", 3)
@@ -177,6 +179,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
         root = self.etree.XML('<a><b>1</b><b>9</b></a>')
         self.check_selector('min(/a/b/number(text()))', root, 1)
+        self.check_selector('min(/a/b)', root, 1)
 
     ###
     # Functions on strings
@@ -185,15 +188,15 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
     def test_string_to_codepoints_function(self):
         self.check_value('string-to-codepoints("Thérèse")', [84, 104, 233, 114, 232, 115, 101])
-        self.check_value('string-to-codepoints(())', [])
+        self.check_value('string-to-codepoints(())')
 
     def test_codepoint_equal_function(self):
         self.check_value("fn:codepoint-equal('abc', 'abc')", True)
         self.check_value("fn:codepoint-equal('abc', 'abcd')", False)
         self.check_value("fn:codepoint-equal('', '')", True)
-        self.check_value("fn:codepoint-equal((), 'abc')", [])
-        self.check_value("fn:codepoint-equal('abc', ())", [])
-        self.check_value("fn:codepoint-equal((), ())", [])
+        self.check_value("fn:codepoint-equal((), 'abc')")
+        self.check_value("fn:codepoint-equal('abc', ())")
+        self.check_value("fn:codepoint-equal((), ())")
 
     def test_compare_function(self):
         env_locale_setting = locale.getlocale(locale.LC_COLLATE)
@@ -271,10 +274,10 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value('fn:normalize-unicode("menù")', 'menù')
         self.wrong_type('fn:normalize-unicode(xs:hexBinary("84"))', 'XPTY0004')
 
-        self.assertRaises(NotImplementedError, self.parser.parse,
+        self.assertRaises(ElementPathValueError, self.parser.parse,
                           'fn:normalize-unicode("à", "FULLY-NORMALIZED")')
         self.wrong_value('fn:normalize-unicode("à", "UNKNOWN")')
-        self.wrong_type('fn:normalize-unicode("à", ())', 'FORG0006', "can't be an empty sequence")
+        self.wrong_type('fn:normalize-unicode("à", ())', 'XPTY0004', "can't be an empty sequence")
 
         # https://www.w3.org/TR/charmod-norm/#normalization_forms
         self.check_value("fn:normalize-unicode('\u01FA')", '\u01FA')
@@ -315,6 +318,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value("fn:count((1, 2, ()))", 2)
         self.check_value("fn:count((((()))))", 0)
         self.check_value("fn:count((((), (), ()), (), (), (), ()))", 0)
+        self.check_value('fn:count((1, 2 to ()))', 1)
         self.check_value("count(('1', (2, ())))", 2)
         self.check_value("count(('1', (2, '3')))", 3)
         self.check_value("count(1 to 5)", 5)
@@ -326,7 +330,10 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
         self.check_value('fn:count((xs:decimal("-999999999999999999")))', 1)
         self.check_value('fn:count((xs:float("0")))', 1)
-        self.check_value("count(//*[@name='John Doe'])", 0)
+
+        self.check_value("count(//*[@name='John Doe'])", MissingContextError)
+        context = XPathContext(self.etree.XML('<root/>'))
+        self.check_value("count(//*[@name='John Doe'])", 0, context)
 
         with self.assertRaises(TypeError) as cm:
             self.check_value("fn:count()")
@@ -402,7 +409,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
         self.check_value("string-join(('a', 'b', 'c'), ', ')", 'a, b, c')
         self.wrong_type("string-join(('a', 'b', 'c'), 8)", 'XPTY0004', '8 is not an instance')
-        self.wrong_type("string-join(('a', 4, 'c'), ', ')", 'FORG0006', 'values must be strings')
+        self.check_value("string-join(('a', 4, 'c'), ', ')", 'a, 4, c')
 
         root = self.etree.XML(XML_GENERIC_TEST)
         self.check_selector("a[string-join((@id, 'foo', 'bar'), ' ') = 'a_id foo bar']",
@@ -499,8 +506,8 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.wrong_value('fn:tokenize("abc", ".*?")', 'FORX0003', 'matches zero-length string')
         self.wrong_value('fn:tokenize("abba", ".?")')
         self.wrong_value('fn:tokenize("abracadabra", "(ab)|(a)", "sxf")')
-        self.wrong_value('fn:tokenize("abracadabra", ())')
-        self.wrong_value('fn:tokenize("abracadabra", "(ab)|(a)", ())')
+        self.wrong_type('fn:tokenize("abracadabra", ())')
+        self.wrong_type('fn:tokenize("abracadabra", "(ab)|(a)", ())')
 
     def test_resolve_uri_function(self):
         self.check_value('fn:resolve-uri("dir1/dir2", "file:///home/")', 'file:///home/dir1/dir2')
@@ -515,7 +522,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.assertEqual(parser.parse('fn:resolve-uri("/dir1/dir2")').evaluate(context),
                          '/dir1/dir2')
         self.assertEqual(parser.parse('fn:resolve-uri("file:text.txt")').evaluate(context),
-                         'http://www.example.com/ns/text.txt')
+                         'file:text.txt')
         self.assertIsNone(parser.parse('fn:resolve-uri(())').evaluate(context))
 
     def test_empty_function(self):
@@ -536,7 +543,9 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value('fn:distinct-values((1, 2.0, 3, 2))', [1, 2.0, 3])
         context = XPathContext(
             root=self.etree.XML('<root/>'),
-            variables={'x': [UntypedAtomic("foo"), UntypedAtomic("bar"), UntypedAtomic("bar")]}
+            variables={
+                'x': [UntypedAtomic("foo"), UntypedAtomic("bar"), UntypedAtomic("bar")]
+            }
         )
         self.check_value('fn:distinct-values($x)', ['foo', 'bar'], context)
 
@@ -555,8 +564,23 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value('fn:index-of ((10, 20, 30, 30, 20, 10), 20)', [2, 5])
         self.check_value('fn:index-of (("a", "sport", "and", "a", "pastime"), "a")', [1, 4])
 
+        # Issue #28
+        root = self.etree.XML("""<root>
+            <incode>030</incode>
+            <descript></descript>
+        </root>""")
+
+        test1 = "/root/descript[index-of(('030','031'), '030')]"
+        test2 = "/root/descript[ancestor::root/incode = '030']"
+        test3 = "/root/descript[index-of(('030','031'), ancestor::root/incode)]"
+
+        self.check_selector(test1, root, [root[1]])
+        self.check_selector(test2, root, [root[1]])
+        self.check_selector(test3, root, [root[1]])
+
     def test_insert_before_function(self):
-        context = XPathContext(root=self.etree.XML('<root/>'), variables={'x': ['a', 'b', 'c']})
+        context = XPathContext(root=self.etree.XML('<root/>'),
+                               variables={'x': ['a', 'b', 'c']})
         self.check_value('fn:insert-before($x, 0, "z")', ['z', 'a', 'b', 'c'], context.copy())
         self.check_value('fn:insert-before($x, 1, "z")', ['z', 'a', 'b', 'c'], context.copy())
         self.check_value('fn:insert-before($x, 2, "z")', ['a', 'z', 'b', 'c'], context.copy())
@@ -564,14 +588,16 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value('fn:insert-before($x, 4, "z")', ['a', 'b', 'c', 'z'], context.copy())
 
     def test_remove_function(self):
-        context = XPathContext(root=self.etree.XML('<root/>'), variables={'x': ['a', 'b', 'c']})
+        context = XPathContext(root=self.etree.XML('<root/>'),
+                               variables={'x': ['a', 'b', 'c']})
         self.check_value('fn:remove($x, 0)', ['a', 'b', 'c'], context)
         self.check_value('fn:remove($x, 1)', ['b', 'c'], context)
         self.check_value('remove($x, 6)', ['a', 'b', 'c'], context)
         self.check_value('fn:remove((), 3)', [])
 
     def test_reverse_function(self):
-        context = XPathContext(root=self.etree.XML('<root/>'), variables={'x': ['a', 'b', 'c']})
+        context = XPathContext(root=self.etree.XML('<root/>'),
+                               variables={'x': ['a', 'b', 'c']})
         self.check_value('reverse($x)', ['c', 'b', 'a'], context)
         self.check_value('fn:reverse(("hello"))', ['hello'], context)
         self.check_value('fn:reverse(())', [])
@@ -604,16 +630,19 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.wrong_value('fn:exactly-one((10, 20, 30, 40))')
 
     def test_qname_function(self):
-        self.check_value('fn:QName("", "person")', 'person')
-        self.check_value('fn:QName((), "person")', 'person')
-        self.check_value('fn:QName("http://www.example.com/ns/", "person")', 'person')
-        self.check_value('fn:QName("http://www.example.com/ns/", "ht:person")', 'ht:person')
+        self.check_value('fn:string(fn:QName("", "person"))', 'person')
+        self.check_value('fn:string(fn:QName((), "person"))', 'person')
+        self.check_value('fn:string(fn:QName("http://www.example.com/ns/", "person"))', 'person')
+        self.check_value('fn:string(fn:QName("http://www.example.com/ns/", "ht:person"))',
+                         'ht:person')
+        self.check_value('fn:string(fn:QName("http://www.example.com/ns/", "xs:person"))',
+                         'xs:person')
 
-        self.wrong_type('fn:QName(1.0, "person")', 'FORG0006', '1st argument has an invalid type')
-        self.wrong_type('fn:QName("", 2)', 'FORG0006', '2nd argument has an invalid type')
-        self.wrong_value('fn:QName("", "3")', 'FOCA0002', '2nd argument must be an xs:QName')
-        self.wrong_value('fn:QName("http://www.example.com/ns/", "xs:person")')
-        self.wrong_value('fn:QName("", "xs:int")', 'FOCA0002', 'must be a local name')
+        self.wrong_type('fn:QName(1.0, "person")', 'XPTY0004', '1st argument has an invalid type')
+        self.wrong_type('fn:QName("", 2)', 'XPTY0004', '2nd argument has an invalid type')
+        self.wrong_value('fn:QName("", "3")', 'FOCA0002', 'invalid value')
+        self.wrong_value('fn:QName("", "xs:int")', 'FOCA0002',
+                         'cannot associate a non-empty prefix with no namespace')
 
     def test_prefix_from_qname_function(self):
         self.check_value(
@@ -622,17 +651,17 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value(
             'fn:prefix-from-QName(fn:QName("http://www.example.com/ns/", "person"))', []
         )
-        self.check_value('fn:prefix-from-QName(())', [])
+        self.check_value('fn:prefix-from-QName(())')
         self.check_value('fn:prefix-from-QName(7)', TypeError)
-        self.check_value('fn:prefix-from-QName("7")', ValueError)
+        self.check_value('fn:prefix-from-QName("7")', TypeError)
 
     def test_local_name_from_qname_function(self):
         self.check_value(
             'fn:local-name-from-QName(fn:QName("http://www.example.com/ns/", "person"))', 'person'
         )
-        self.check_value('fn:local-name-from-QName(())', [])
+        self.check_value('fn:local-name-from-QName(())')
         self.check_value('fn:local-name-from-QName(8)', TypeError)
-        self.check_value('fn:local-name-from-QName("8")', ValueError)
+        self.check_value('fn:local-name-from-QName("8")', TypeError)
 
     def test_namespace_uri_from_qname_function(self):
         root = self.etree.XML('<p1:A xmlns:p1="ns1" xmlns:p0="ns0">'
@@ -643,11 +672,11 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
             'fn:namespace-uri-from-QName(fn:QName("http://www.example.com/ns/", "person"))',
             'http://www.example.com/ns/'
         )
-        self.check_value('fn:namespace-uri-from-QName(())', [])
+        self.check_value('fn:namespace-uri-from-QName(())')
         self.check_value('fn:namespace-uri-from-QName(1)', TypeError)
-        self.check_value('fn:namespace-uri-from-QName("1")', ValueError)
-        self.check_selector("fn:namespace-uri-from-QName('p3:C3')", root, KeyError)
-        self.check_selector("fn:namespace-uri-from-QName('p3:C3')", root, NameError,
+        self.check_value('fn:namespace-uri-from-QName("1")', TypeError)
+        self.check_selector("fn:namespace-uri-from-QName(xs:QName('p3:C3'))", root, KeyError)
+        self.check_selector("fn:namespace-uri-from-QName(xs:QName('p3:C3'))", root, ValueError,
                             namespaces={'p3': ''})
 
     def test_resolve_qname_function(self):
@@ -657,28 +686,17 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
                               '</p1:A>')
         context = XPathContext(root=root)
 
-        self.check_value("fn:resolve-QName((), .)", [], context=context.copy())
-        self.check_value("fn:resolve-QName('eg:C2', .)", '{http://www.example.com/ns/}C2',
-                         context=context.copy())
-        self.check_selector("fn:resolve-QName('p3:C3', .)", root, NameError, namespaces={'p3': ''})
+        self.check_value("fn:resolve-QName((), .)", context=context.copy())
+        self.check_value("fn:string(fn:resolve-QName('eg:C2', .))", 'eg:C2', context=context.copy())
+        self.check_selector("fn:resolve-QName('p3:C3', .)", root, ValueError, namespaces={'p3': ''})
         self.check_selector("fn:resolve-QName('p3:C3', .)", root, KeyError)
         self.check_value("fn:resolve-QName(2, .)", TypeError, context=context.copy())
         self.check_value("fn:resolve-QName('2', .)", ValueError, context=context.copy())
-        self.check_value("fn:resolve-QName((), 4)", TypeError, context=context.copy())
+        self.check_value("fn:resolve-QName((), 4)", context=context.copy())
 
         root = self.etree.XML('<A><B1><C/></B1><B2/><B3><C1/><C2/></B3></A>')
-        self.check_selector("fn:resolve-QName('C3', .)", root, ['C3'], namespaces={'': ''})
-
-        if xmlschema is not None:
-            schema = xmlschema.XMLSchema("""
-                <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" 
-                    xmlns:tns="http://foo.test">
-                  <xs:element name="root"/>
-                </xs:schema>""")
-
-            with self.schema_bound_parser(schema.xpath_proxy):
-                context = self.parser.schema.get_context()
-                self.check_value("fn:resolve-QName('p3:C3', .)", None, context)
+        self.check_selector("fn:resolve-QName('C3', .)", root,
+                            [QName('', 'C3')], namespaces={'': ''})
 
     def test_namespace_uri_for_prefix_function(self):
 
@@ -688,7 +706,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
                               '</p1:A>')
         context = XPathContext(root=root)
 
-        self.check_value("fn:namespace-uri-for-prefix('p1', .)", [], context=context.copy())
+        self.check_value("fn:namespace-uri-for-prefix('p1', .)", context=context.copy())
         self.check_value("fn:namespace-uri-for-prefix(4, .)", TypeError, context=context.copy())
         self.check_value("fn:namespace-uri-for-prefix('p1', 9)", TypeError, context=context.copy())
         self.check_value("fn:namespace-uri-for-prefix('eg', .)",
@@ -697,15 +715,13 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
                             root, NameError, namespaces={'p3': ''})
 
         # Note: default namespace for XPath 2 tests is 'http://www.example.com/ns/'
-        self.check_value("fn:namespace-uri-for-prefix('', .)", [], context=context.copy())
+        self.check_value("fn:namespace-uri-for-prefix('', .)", context=context.copy())
         self.check_value(
             'fn:namespace-uri-from-QName(fn:QName("http://www.example.com/ns/", "person"))',
             'http://www.example.com/ns/'
         )
-        self.check_value("fn:namespace-uri-for-prefix('', .)",
-                         'http://www.example.com/ns/', context=context.copy())
-        self.check_value("fn:namespace-uri-for-prefix((), .)",
-                         'http://www.example.com/ns/', context=context.copy())
+        self.check_value("fn:namespace-uri-for-prefix('', .)", context=context.copy())
+        self.check_value("fn:namespace-uri-for-prefix((), .)", context=context.copy())
 
     def test_in_scope_prefixes_function(self):
         root = self.etree.XML('<p1:A xmlns:p1="ns1" xmlns:p0="ns0">'
@@ -743,8 +759,8 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         tz0 = None
         tz1 = Timezone(datetime.timedelta(hours=5, minutes=24))
 
-        self.check_value('fn:dateTime((), xs:time("24:00:00"))')
-        self.check_value('fn:dateTime(xs:date("1999-12-31"), ())')
+        self.check_value('fn:dateTime((), xs:time("24:00:00"))', [])
+        self.check_value('fn:dateTime(xs:date("1999-12-31"), ())', [])
         self.check_value('fn:dateTime(xs:date("1999-12-31"), xs:time("12:00:00"))',
                          datetime.datetime(1999, 12, 31, 12, 0, tzinfo=tz0))
         self.check_value('fn:dateTime(xs:date("1999-12-31"), xs:time("24:00:00"))',
@@ -824,52 +840,52 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
     def test_seconds_from_time_function(self):
         self.check_value('fn:seconds-from-time(xs:time("13:20:10.5"))', 10.5)
         self.check_value('fn:seconds-from-time(xs:time("20:50:10.0"))', 10.0)
-        self.check_value('fn:seconds-from-time(xs:time("03:59:59.000001"))', 59.000001)
+        self.check_value('fn:seconds-from-time(xs:time("03:59:59.000001"))', Decimal('59.000001'))
 
     def test_timezone_from_time_function(self):
         self.check_value('fn:timezone-from-time(xs:time("13:20:00-05:00"))',
                          DayTimeDuration.fromstring('-PT5H'))
 
     def test_years_from_duration_function(self):
-        self.check_value('fn:years-from-duration(())', [])
+        self.check_value('fn:years-from-duration(())')
         self.check_value('fn:years-from-duration(xs:yearMonthDuration("P20Y15M"))', 21)
         self.check_value('fn:years-from-duration(xs:yearMonthDuration("-P15M"))', -1)
         self.check_value('fn:years-from-duration(xs:dayTimeDuration("-P2DT15H"))', 0)
 
     def test_months_from_duration_function(self):
-        self.check_value('fn:months-from-duration(())', [])
+        self.check_value('fn:months-from-duration(())')
         self.check_value('fn:months-from-duration(xs:yearMonthDuration("P20Y15M"))', 3)
         self.check_value('fn:months-from-duration(xs:yearMonthDuration("-P20Y18M"))', -6)
         self.check_value('fn:months-from-duration(xs:dayTimeDuration("-P2DT15H0M0S"))', 0)
 
     def test_days_from_duration_function(self):
-        self.check_value('fn:days-from-duration(())', [])
+        self.check_value('fn:days-from-duration(())')
         self.check_value('fn:days-from-duration(xs:dayTimeDuration("P3DT10H"))', 3)
         self.check_value('fn:days-from-duration(xs:dayTimeDuration("P3DT55H"))', 5)
         self.check_value('fn:days-from-duration(xs:yearMonthDuration("P3Y5M"))', 0)
 
     def test_hours_from_duration_function(self):
-        self.check_value('fn:hours-from-duration(())', [])
+        self.check_value('fn:hours-from-duration(())')
         self.check_value('fn:hours-from-duration(xs:dayTimeDuration("P3DT10H"))', 10)
         self.check_value('fn:hours-from-duration(xs:dayTimeDuration("P3DT12H32M12S"))', 12)
         self.check_value('fn:hours-from-duration(xs:dayTimeDuration("PT123H"))', 3)
         self.check_value('fn:hours-from-duration(xs:dayTimeDuration("-P3DT10H"))', -10)
 
     def test_minutes_from_duration_function(self):
-        self.check_value('fn:minutes-from-duration(())', [])
+        self.check_value('fn:minutes-from-duration(())')
         self.check_value('fn:minutes-from-duration(xs:dayTimeDuration("P3DT10H"))', 0)
         self.check_value('fn:minutes-from-duration(xs:dayTimeDuration("-P5DT12H30M"))', -30)
 
     def test_seconds_from_duration_function(self):
-        self.check_value('fn:seconds-from-duration(())', [])
+        self.check_value('fn:seconds-from-duration(())')
         self.check_value('fn:seconds-from-duration(xs:dayTimeDuration("P3DT10H12.5S"))', 12.5)
         self.check_value('fn:seconds-from-duration(xs:dayTimeDuration("-PT256S"))', -16.0)
 
     def test_node_accessor_functions(self):
         root = self.etree.XML('<A xmlns:ns0="%s" id="10"><B1><C1 /><C2 ns0:nil="true" /></B1>'
                               '<B2 /><B3>simple text</B3></A>' % XSI_NAMESPACE)
-        self.check_selector("node-name(.)", root, 'A')
-        self.check_selector("node-name(/A/B1)", root, 'B1')
+        self.check_selector("node-name(.)", root, QName('', 'A'))
+        self.check_selector("node-name(/A/B1)", root, QName('', 'B1'))
         self.check_selector("node-name(/A/*)", root, TypeError)  # Not allowed more than one item!
         self.check_selector("nilled(./B1/C1)", root, False)
         self.check_selector("nilled(./B1/C2)", root, True)
@@ -884,17 +900,12 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
         context = XPathContext(root=self.etree.XML('<A/>'))
         parser = XPath2Parser(base_uri='http://www.example.com/ns/')
-        with self.assertRaises(ValueError) as err:
-            parser.parse('data(fn:resolve-uri(()))').evaluate(context)
-        self.assertIn('FOTY0012', str(err.exception))
-        self.assertIn('argument node does not have a typed value', str(err.exception))
+        self.assertEqual(parser.parse('data(fn:resolve-uri(()))').evaluate(context), [])
 
     def test_node_set_id_function(self):
-        # Backward compatibility with fs:id() of XPath 1
         root = self.etree.XML('<A><B1 xml:id="foo"/><B2/><B3 xml:id="bar"/><B4 xml:id="baz"/></A>')
         self.check_selector('element-with-id("foo")', root, [root[0]])
-
-        self.check_selector('id("foo")', root, ValueError)
+        self.check_selector('id("foo")', root, [root[0]])
 
         doc = self.etree.parse(
             io.StringIO('<A><B1 xml:id="foo"/><B2/><B3 xml:id="bar"/><B4 xml:id="baz"/></A>')
@@ -915,17 +926,17 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
                <last>Brown</last>
             </employee>"""))
         root = doc.getroot()
+
         self.check_selector("id('ID21256')", doc, [root])
         self.check_selector("id('E21256')", doc, [root[0]])
-
-        with self.assertRaises(ValueError) as err:
-            self.check_selector("id('ID21256')", root, [root])
-        self.assertIn('FODC0001', str(err.exception))
+        with self.assertRaises(MissingContextError) as err:
+            self.check_value("id('ID21256')")
+        self.assertIn('XPDY0002', str(err.exception))
 
         context = XPathContext(doc, variables={'x': 11})
-        with self.assertRaises(ValueError) as err:
+        with self.assertRaises(TypeError) as err:
             self.check_value("id('ID21256', $x)", context=context)
-        self.assertIn('FODC0001', str(err.exception))
+        self.assertIn('XPTY0004', str(err.exception))
 
         context = XPathContext(doc, item=11, variables={'x': 11})
         with self.assertRaises(TypeError) as err:
@@ -954,15 +965,12 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value("idref('ID21256')", MissingContextError)
         self.check_selector("idref('ID21256')", doc, [])
         self.check_selector("idref('E21256')", doc, [root[0][0]])
-
-        with self.assertRaises(ValueError) as err:
-            self.check_selector("idref('ID21256')", root, [root])
-        self.assertIn('FODC0001', str(err.exception))
+        self.check_selector("idref('ID21256')", root, [])
 
         context = XPathContext(doc, variables={'x': 11})
-        with self.assertRaises(ValueError) as err:
+        with self.assertRaises(TypeError) as err:
             self.check_value("idref('ID21256', $x)", context=context)
-        self.assertIn('FODC0001', str(err.exception))
+        self.assertIn('XPTY0004', str(err.exception))
 
         context = XPathContext(doc, item=11, variables={'x': 11})
         with self.assertRaises(TypeError) as err:
@@ -1030,6 +1038,8 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
             DateTime.fromstring('2002-03-07T10:00:00'), context
         )
 
+        self.check_value('fn:adjust-dateTime-to-timezone((), ())')
+
     def test_adjust_date_to_timezone_function(self):
         context = XPathContext(root=self.etree.XML('<A/>'), timezone=Timezone.fromstring('-05:00'),
                                variables={'tz': DayTimeDuration.fromstring("-PT10H")})
@@ -1047,11 +1057,16 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value('fn:adjust-date-to-timezone(xs:date("2002-03-07-07:00"), $tz)',
                          Date.fromstring('2002-03-06-10:00'), context)
 
+        self.check_value('fn:adjust-date-to-timezone((), ())')
+        self.check_value('adjust-date-to-timezone(xs:date("-25252734927766555-06-07+02:00"), '
+                         'xs:dayTimeDuration("PT0S"))', OverflowError)
+
     def test_adjust_time_to_timezone_function(self):
         context = XPathContext(root=self.etree.XML('<A/>'), timezone=Timezone.fromstring('-05:00'),
                                variables={'tz': DayTimeDuration.fromstring("-PT10H")})
 
         self.check_value('fn:adjust-time-to-timezone(())')
+        self.check_value('fn:adjust-time-to-timezone((), ())')
 
         self.check_value('fn:adjust-time-to-timezone(xs:time("10:00:00"))',
                          Time.fromstring('10:00:00-05:00'), context)
@@ -1070,9 +1085,8 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
                          Time.fromstring('03:00:00+10:00'), context)
 
     def test_default_collation_function(self):
-        default_locale = locale.getdefaultlocale()
-        default_locale = '.'.join(default_locale) if default_locale[1] else default_locale[0]
-        self.check_value('fn:default-collation()', default_locale)
+        default_collation = self.parser.default_collation
+        self.check_value('fn:default-collation()', default_collation)
 
     def test_context_functions(self):
         context = XPathContext(root=self.etree.XML('<A/>'))
@@ -1084,10 +1098,10 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value(path='fn:current-time()', context=context,
                          expected=Time.fromdatetime(context.current_dt),)
         self.check_value(path='fn:implicit-timezone()', context=context,
-                         expected=Timezone(datetime.timedelta(seconds=time.timezone)),)
+                         expected=DayTimeDuration(seconds=time.timezone))
         context.timezone = Timezone.fromstring('-05:00')
         self.check_value(path='fn:implicit-timezone()', context=context,
-                         expected=Timezone.fromstring('-05:00'))
+                         expected=DayTimeDuration.fromstring('-PT5H'))
 
     def test_static_base_uri_function(self):
         context = XPathContext(root=self.etree.XML('<A/>'))
@@ -1104,7 +1118,8 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.assertIn('XPDY0002', str(err.exception))
         self.assertIn('context item is undefined', str(err.exception))
 
-        self.wrong_type('fn:base-uri(9)', 'XPTY0004', 'context item is not a node')
+        self.check_value('fn:base-uri(9)', MissingContextError)
+        self.check_value('fn:base-uri(9)', TypeError, context=context)
         self.check_value('fn:base-uri()', context=context)
 
         context = XPathContext(root=self.etree.XML('<A xml:base="/base_path/"/>'))
@@ -1112,7 +1127,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
     def test_document_uri_function(self):
         context = XPathContext(root=self.etree.parse(io.StringIO('<A/>')))
-        self.check_value('fn:document-uri(())', [], context=context)
+        self.check_value('fn:document-uri(())', context=context)
         self.check_value('fn:document-uri(.)', context=context)
 
         context = XPathContext(root=self.etree.parse(io.StringIO('<A xml:base="/base_path/"/>')))
@@ -1140,13 +1155,13 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         doc = self.etree.XML("<a><b1><c1/></b1><b2/><b3/></a>")
         context = XPathContext(root, documents={'tns0': doc})
 
-        with self.assertRaises(ValueError) as err:
+        with self.assertRaises(TypeError) as err:
             self.check_value("fn:doc('tns0')", context=context)
-        self.assertIn('FODC0005', str(err.exception))
+        self.assertIn('XPDY0050', str(err.exception))
 
-        with self.assertRaises(ValueError) as err:
+        with self.assertRaises(TypeError) as err:
             self.check_value("fn:doc-available('tns0')", context=context)
-        self.assertIn('FODC0005', str(err.exception))
+        self.assertIn('XPDY0050', str(err.exception))
 
     def test_collection_function(self):
         root = self.etree.XML("<A><B1><C1/></B1><B2/><B3/></A>")
@@ -1154,11 +1169,19 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         doc2 = self.etree.parse(io.StringIO("<a1><b11><c11/></b11><b12/><b13/></a1>"))
         context = XPathContext(root, collections={'tns0': [doc1, doc2]})
 
+        self.check_value("fn:collection('tns0')", [doc1, doc2], context=context)
+
+        self.parser.collection_types = {'tns0': 'node()*'}
+        self.check_value("fn:collection('tns0')", [doc1, doc2], context=context)
+        self.parser.collection_types = {'tns0': 'node()'}
+        self.check_value("fn:collection('tns0')", TypeError, context=context)
+
         self.check_value("fn:collection()", ValueError, context=context)
-        self.check_value("fn:collection('tns0')", ValueError, context=context)
         context.default_collection = context.collections['tns0']
         self.check_value("fn:collection()", [doc1, doc2], context=context)
-        self.check_value("fn:collection('tns0')", ValueError, context=context)
+        self.parser.default_collection_type = 'node()'
+        self.check_value("fn:collection()", TypeError, context=context)
+        self.parser.default_collection_type = 'node()*'
 
     def test_root_function(self):
         root = self.etree.XML("<A><B1><C1/></B1><B2/><B3/></A>")
@@ -1204,15 +1227,15 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
     def test_error_function(self):
         with self.assertRaises(ElementPathError) as err:
             self.check_value('fn:error()')
-        self.assertEqual(str(err.exception), '[err:FOER0000] Unidentified error.')
+        self.assertEqual(str(err.exception), '[err:FOER0000] Unidentified error')
 
         with self.assertRaises(ElementPathError) as err:
             self.check_value('fn:error("err:XPST0001")')
-        self.assertEqual(str(err.exception), '[err:XPST0001] Parser not bound to a schema.')
+        self.assertEqual(str(err.exception), '[err:XPST0001] Parser not bound to a schema')
 
         with self.assertRaises(ElementPathError) as err:
             self.check_value('fn:error("err:XPST0001", "Missing schema")')
-        self.assertEqual(str(err.exception), '[err:XPST0001] Missing schema.')
+        self.assertEqual(str(err.exception), '[err:XPST0001] Missing schema')
 
 
 @unittest.skipIf(lxml_etree is None, "The lxml library is not installed")
